@@ -29,7 +29,12 @@ class CommandCompletion(Completion):
             if cmd is not None and isinstance(targetpanel, cmd.ACCEPT_IN):
                 alternatives = cmd().completion_options(self.app.make_context(), *parts[1:])
 
-        if len(parts) > 0 and len(alternatives) == 1 and alternatives[0] == parts[-1]:
+        only_alternative_typed_out = len(parts) > 0 and \
+                                     len(alternatives) == 1 and \
+                                     ((isinstance(alternatives[0], Completion.Suggestion) \
+                                       and alternatives[0].text == parts[-1]) or \
+                                      (alternatives[0] == parts[-1]))
+        if only_alternative_typed_out:
             self.close()
             self.app.paint()
         elif len(alternatives) > 0:
@@ -123,7 +128,7 @@ class CommandInput(InputLine):
             self.history_cursor = None
             self.cached_text = None
             self.cached_cursor = None
-        
+
         else:
             text = self.text
             super().handle_key(key)
