@@ -136,7 +136,7 @@ class EnterCommandMode(Command):
         context.application.previous_focus = context.panel
         context.application.activate_panel(context.application.command_input)
         context.application.paint()
-        logger.debug(f"Enter command input")
+        logger.debug("Enter command input")
 
 
 @registered_command
@@ -176,7 +176,7 @@ class EditMetadataExternally(Command):
 
         # ensure there is something to edit
         if file is None:
-            logger.debug(f"No file to edit")
+            logger.debug("No file to edit")
             context.application.error("Nothing selected")
             return
 
@@ -291,47 +291,6 @@ class SelectFileAndExit(Command):
         context.application.execute_command('quit')
 
 
-@registered_command
-class CopyPathToClipboard(Command):
-    """Copy the selected item to clipboard"""
-    NAME = 'copy'
-    ACCEPT_IN = (DocPanel, FilePanel)
-
-    def execute(self, context, clipboard=None):
-        source = context.panel
-        if source.is_busy:
-            return
-
-        context.application.clear_clipboard(clipboard)
-        for path in source.selected_paths:
-            context.application.append_to_clipboard(path, clipboard)
-
-
-@registered_command
-class AppendPathToClipboard(Command):
-    """Append the selected item path to clipboard"""
-    NAME = 'append'
-    ACCEPT_IN = (DocPanel, FilePanel)
-
-    def execute(self, context, clipboard=None):
-        source = context.panel
-        if source.is_busy:
-            return
-
-        for path in source.selected_paths:
-            context.application.append_to_clipboard(path, clipboard)
-
-
-@registered_command
-class ClearClipboard(Command):
-    """Clear the clipboard of items to copy"""
-    NAME = 'clear-clipboard'
-
-    def execute(self, context, clipboard=None):
-        context.application.clear_clipboard(clipboard)
-        context.application.info(f"Clipboard {clipboard} cleared")
-
-
 @registered_command
 class RefreshPanel(Command):
     """Refresh the current panel"""
@@ -368,6 +327,7 @@ class DeleteItem(Command):
 class BookmarkNameReader(InputLine):
     def __init__(self, application, prefix):
         height, width = application.size()
+        application.clear_info_text()
         super().__init__(application, width, (height-1, 0), prefix=prefix)
 
     def handle_key(self, key):
@@ -537,16 +497,16 @@ class SetCommand(Command):
     NAME = 'set'
 
     def completion_options(self, context, *args):
-        text = args[0] if len(args) > 0 else ""
-        scope = ""
-        name = ""
+        logger.debug("Completion base: %s", args)
 
         options = []
-        for group in sorted(context.application.configuration.conf.sections()):
-            for option in sorted(context.application.configuration.conf[group]):
-                full = group + "." + option
-                if full.startswith(text):
-                    options.append(group + "." + option)
+        if len(args) < 2:
+            text = args[0] if len(args) > 0 else ""
+            for group in sorted(context.application.configuration.conf.sections()):
+                for option in sorted(context.application.configuration.conf[group]):
+                    full = group + "." + option
+                    if full.startswith(text):
+                        options.append(group + "." + option)
 
         return options
 
@@ -598,7 +558,8 @@ class ClearSelection(Command):
         if context.panel.is_busy:
             return
 
-        indexes = [idx for idx, item in enumerate(context.panel.items) if item in context.panel.multi_selection]
+        indexes = [idx for idx, item in enumerate(context.panel.items)
+                   if item in context.panel.multi_selection]
         context.panel.multi_selection = []
         for index in indexes:
             context.panel.paint_item(index)
@@ -614,7 +575,8 @@ class InvertSelection(Command):
         if context.panel.is_busy:
             return
 
-        context.panel.multi_selection = [item for item in context.panel.items if item not in context.panel.multi_selection]
+        context.panel.multi_selection = [item for item in context.panel.items
+                                         if item not in context.panel.multi_selection]
         context.panel.paint()
 
 
@@ -676,16 +638,18 @@ class RunIndexers(Command):
             return
 
         path = context.panel.selected_path
-        context.panel.run_blocking(self.run_indexers, context, path)
+        context.panel.run_blocking(self.run_indexers, context.application, context.panel, path)
         context.application.paint(True)
 
-    def run_indexers(self, blocker, context, path):
+    @staticmethod
+    def run_indexers(blocker, application, panel, path):
         blocker.title(f"Running indexers on {path.name}")
+        logger.debug("Running indexers on %s", path.name)
 
-        item = context.panel.selected_item
+        item = panel.selected_item
 
         if path.is_dir():
-            paths = context.application.cache.find_indexable_files([path])
+            paths = application.cache.find_indexable_files([path])
         else:
             paths = [path]
 
@@ -693,7 +657,7 @@ class RunIndexers(Command):
                                                 processes=1,
                                                 ocr_=metaindex.ocr.Dummy(),
                                                 fulltext=False,
-                                                config=context.application.metaindexconf)
+                                                config=application.metaindexconf)
         if len(results) == 0:
             return
 
@@ -703,7 +667,7 @@ class RunIndexers(Command):
         for idx, result in enumerate(results):
             lpath, success, extra = result
 
-            base = context.application.cache.get(lpath, False)
+            base = application.cache.get(lpath, False)
             if len(base) == 0:
                 info = multidict.MultiDict()
             else:
@@ -727,13 +691,13 @@ class RunIndexers(Command):
                 logger.debug("Nothing new here")
                 return
 
-            context.application.cache.insert(lpath, info)
+            application.cache.insert(lpath, info)
             blocker.progress((idx+1, len(results)))
 
-        if isinstance(context.panel, (DocPanel,)):
-            context.panel.search(context.panel.query)
-            context.application.callbacks.put((context.panel,
-                                               lambda: context.panel.jump_to(item)))
-        elif isinstance(context.panel, (EditorPanel,)):
-            context.application.callbacks.put((context.panel,
-                                               lambda: context.panel.reload()))
+        if isinstance(panel, (DocPanel,)):
+            panel.search(panel.query)
+            application.callbacks.put((panel,
+                                       lambda: panel.jump_to(item)))
+        elif isinstance(panel, (EditorPanel,)):
+            application.callbacks.put((panel,
+                                       panel.reload))
