@@ -11,13 +11,14 @@ import metaindex.indexer
 
 from cursedspace import Key, InputLine, ShellContext
 
+from metaindexmanager import clipboard
 from metaindexmanager import command
 from metaindexmanager import utils
-from metaindexmanager.utils import logger
+from metaindexmanager.utils import logger, first_line
 from metaindexmanager.panel import ListPanel
-from metaindexmanager.detailpanel import DetailPanel
 from metaindexmanager.docpanel import DocPanel
 from metaindexmanager.filepanel import FilePanel
+from metaindexmanager.detailpanel import DetailPanel
 
 
 Change = collections.namedtuple("Change", ['index', 'new_value', 'prefix', 'tag', 'old_value'])
@@ -90,6 +91,8 @@ class EditorPanel(ListPanel):
         self.editor = None
         self.metadata = multidict.MultiDict()
 
+        self.auto_index_if_not_in_cache = True
+
         self._multiline_icon = ' '
         self._cutoff_icon = ' '
         self.configuration_changed()
@@ -109,12 +112,36 @@ class EditorPanel(ListPanel):
     def selected_item(self):
         return str(self.item)
 
+    @property
+    def selected_items(self):
+        if len(self.multi_selection) == 0:
+            line = self.selected_line
+            if isinstance(line, Line):
+                return [line]
+            return []
+        return self.multi_selection
+
     @property
     def selected_line(self):
         if 0 <= self.cursor < len(self.items):
             return self.items[self.cursor]
         return None
 
+    def on_copy(self, item):
+        return clipboard.ClipboardItem(item, self)
+
+    def on_paste(self, items):
+        if self.is_busy:
+            raise RuntimeError("Cannot paste right now, editor is busy")
+
+        changes = [Insert('extra', item.data.tag, item.data.value)
+                   for item in items if isinstance(item.data, Line)]
+        if len(changes) == 0:
+            raise RuntimeError("Cannot paste this here")
+
+        grouped = GroupedChange(changes)
+        self.changed(grouped)
+
     def open_selected(self):
         if self.item is None:
             return
@@ -313,10 +340,18 @@ class EditorPanel(ListPanel):
 
     def reload(self):
         logger.debug("Reloading %s", self.item)
+
         metadata = [entry.metadata
                     for entry in self.app.cache.get(self.item)
                     if entry.path == self.item]
 
+        if len(metadata) == 0 and self.auto_index_if_not_in_cache:
+            self.auto_index_if_not_in_cache = False
+            index_cmd = command.resolve_command('index')
+            if index_cmd is not None:
+                self.run_blocking(index_cmd.run_indexers, self.app, self, self.item)
+                return
+
         if len(metadata) == 0:
             self.metadata = multidict.MultiDict({'filename': str(self.selected_path.name)})
         else:
@@ -400,7 +435,7 @@ class EditorPanel(ListPanel):
 
                     # multi-line is special
                     is_multiline = '\r' in text or '\n' in text
-                    text = utils.first_line(text)
+                    text = first_line(text)
                     if is_multiline:
                         text += ' ' + self._multiline_icon
 
@@ -679,58 +714,6 @@ class UndoAllChanges(command.Command):
         context.panel.undo()
 
 
-@command.simple_command("copy-tag", (EditorPanel,))
-def copy_tag_command(context, clipboard=None):
-    """Copy the selected tag and value to clipboard"""
-    source = context.panel
-    if source.is_busy:
-        return
-
-    context.application.clear_clipboard(clipboard)
-
-    line = source.selected_line
-
-    if line is None or isinstance(line, Header):
-        return
-
-    context.application.append_to_clipboard((line,), clipboard)
-
-
-@command.simple_command("copy-append-tag", (EditorPanel,))
-def copy_append_tag_command(context, clipboard=None):
-    """Add the selected tag and value to clipboard"""
-    source = context.panel
-    if source.is_busy:
-        return
-
-    line = source.selected_line
-
-    if line is None or isinstance(line, Header):
-        return
-
-    context.application.append_to_clipboard((line,), clipboard)
-
-
-@command.simple_command("paste-tag", (EditorPanel,))
-def paste_tag_command(context, clipboard=None):
-    """Paste tag and value from clipboard"""
-    target = context.panel
-    if target.is_busy:
-        return
-
-    content = context.application.get_clipboard_content(clipboard)
-    if content is None:
-        return
-
-    items = [item for item in content if isinstance(item, (Line,))]
-    if len(items) == 0:
-        return
-
-    changes = [Insert('extra', line.tag, line.value) for line in items]
-    grouped = GroupedChange(changes)
-    target.changed(grouped)
-
-
 @command.registered_command
 class RunRules(command.Command):
     """Run tag rules on this document"""
