@@ -5,13 +5,16 @@ import shutil
 import threading
 import subprocess
 
-from cursedspace import Panel, colors, ShellContext
+from multidict import MultiDict
+
+from cursedspace import Panel, colors, ShellContext, Completion
 
 from metaindex import stores
-from metaindex.shared import IS_RECURSIVE
+from metaindex.configuration import SECTION_GENERAL, CONFIG_COLLECTION_METADATA
 
 from metaindexmanager import command
 from metaindexmanager import utils
+from metaindexmanager import clipboard
 from metaindexmanager.utils import logger
 from metaindexmanager.panel import ListPanel
 
@@ -111,6 +114,12 @@ class FilePanel(ListPanel):
         self.configuration_changed()
         self.change_path(path)
 
+        # when copying, moving, or deleting files, do not attempt to move
+        # sidecar files for sidecar files, e.g. copying a 'metadata.yaml'
+        # should not result in the adjacent 'metadata.json' to be copied, too.
+        self.metaindexfilenames = self.app.metaindexconf.list(SECTION_GENERAL,
+                                                              CONFIG_COLLECTION_METADATA)
+
     def configuration_changed(self, name=None):
         super().configuration_changed(name)
 
@@ -128,16 +137,17 @@ class FilePanel(ListPanel):
 
     def change_path(self, path):
         logger.debug(f"FilePanel: changing to path {path}")
-        if self.path is not None:
-            self.cursor_cache[self.path] = self.cursor
-        self.previous_path = self.path
         path = pathlib.Path(path).expanduser().resolve()
-        if path != self.path and self.observer is not None:
+        if self.observer is not None:
             self.observer.active = False
             self.observer = None
-        self.path = path
-        self.offset = 0
-        self.cursor = 0
+        if self.path != path:
+            if self.path is not None:
+                self.cursor_cache[self.path] = self.cursor
+            self.previous_path = self.path
+            self.path = path
+            self.offset = 0
+            self.cursor = 0
         self.items = None
         if self.observer is None:
             self.observer = FolderObserver(self, path)
@@ -145,7 +155,7 @@ class FilePanel(ListPanel):
 
     def finish_change_path(self, items):
         self.items = [i.copy() for i in sorted(items, key=lambda f: [not f.is_dir, f.name])
-                      if self.show_hidden_files or not i.name.startswith('.')]
+                      if self.show_hidden_files or not utils.is_hidden(i)]
 
         if self.path in self.cursor_cache:
             self.cursor = self.cursor_cache[self.path]
@@ -178,7 +188,7 @@ class FilePanel(ListPanel):
 
     @property
     def selected_paths(self):
-        return [i.fullpath for i in self.selected_items]
+        return [i.fullpath for i in self.selected_items if i is not None]
 
     def open_selected(self):
         item = self.selected_item
@@ -193,6 +203,186 @@ class FilePanel(ListPanel):
         else:
             self.app.open_file(item.fullpath)
 
+    def on_clipboard_item_use(self, item):
+        """Called when a clipboard item copied/cut from here is being pasted"""
+        if item.extra != 'CUT':
+            return
+        # drop the item from clipboard when it was a 'cut' operation
+        item.valid = False
+
+    def on_cut(self, item):
+        if self.is_busy:
+            raise RuntimeError("Cannot cut right now")
+        if not os.access(item.fullpath, os.R_OK):
+            raise RuntimeError(f"File '{item.fullpath}' can not be read")
+        return clipboard.ClipboardItem(item, self, 'CUT')
+
+    def on_copy(self, item):
+        if self.is_busy:
+            raise RuntimeError("Cannot copy right now")
+        if not os.access(item.fullpath, os.R_OK):
+            raise RuntimeError(f"File '{item.fullpath}' can not be read")
+        return clipboard.ClipboardItem(item, self)
+
+    def on_paste(self, items):
+        if self.is_busy:
+            raise RuntimeError("Cannot paste. Busy")
+
+        paths = []
+        for item in items:
+            is_cut = item.extra == 'CUT'
+            if isinstance(item.data, str):
+                item = pathlib.Path(item.data)
+            elif isinstance(item.data, Line):
+                item = item.data.path
+            elif isinstance(item.data, pathlib.Path):
+                item = item.data
+            else:
+                raise RuntimeError(f"Cannot paste a {type(item.data).__name__}")
+            paths.append((item, is_cut))
+
+        self.run_blocking(self.do_paste, paths)
+
+    def do_paste(self, blocker, paths):
+        if blocker is not None:
+            blocker.title("Copying")
+
+        # copy the files here
+        copied_files = []
+        copied_directories = []
+        moved_files = []
+        moved_directories = []
+        for path, is_cut in paths:
+            extension = ''
+            while True:
+                new_fname = path.stem + extension + path.suffix
+                if not (self.path / new_fname).exists():
+                    break
+                if len(extension) == 0:
+                    extension = '_1'
+                else:
+                    extension = '_' + str(int(extension[1:])+1)
+            new_path = self.path / new_fname
+            if is_cut:
+                # move
+                try:
+                    is_dir = path.is_dir()
+                    shutil.move(path, new_path)
+                    if is_dir:
+                        moved_directories.append((path, new_path))
+                    elif path.stem not in self.metaindexfilenames:
+                        moved_files.append((path, new_path))
+                except OSError as exc:
+                    self.app.error(f"Failed to move {path.name}: {exc}")
+            else:
+                # only copy
+                try:
+                    if path.is_dir():
+                        shutil.copytree(path, new_path)
+                        copied_directories.append(new_path)
+                    elif path.stem:
+                        shutil.copy2(path, new_path)
+                        if path.stem not in self.metaindexfilenames:
+                            copied_files.append((path, new_path))
+                except OSError as exc:
+                    self.app.error(f"Failed to copy {path.name}: {exc}")
+
+        # TODO - do this really by copying the information in the database
+        # self.app.cache.refresh(copied_directories)
+        logger.debug("Copied files: %s", copied_files)
+
+        preferred_stores = self.app.metaindexconf.get_preferred_sidecar_stores()
+
+        # obtain any cached 'extra' metadata and write it into sidecar files
+        inserts = []
+        # in case the transfer of metadata/sidecar files fails, make sure any
+        # deletions are *not* carried out
+        prevent_deletions = False
+
+        for old_path, new_path in copied_files + moved_files:
+            entries = self.app.cache.get(old_path)
+            if len(entries) == 0:
+                continue
+            metadata = entries[0].metadata
+            last_modified = entries[0].last_modified
+            inserts.append((new_path, metadata, last_modified))
+
+            # get all extra metadata, but strip the 'extra.' prefix
+            writeable_metadata = MultiDict([(k.split('.', 1)[1], v)
+                                            for k, v in metadata.items()
+                                            if k.startswith('extra.')])
+
+            # if there is a direct sidecar file, the user probably wants
+            # a direct sidecar file again for the new file
+            direct_sidecars = [fn
+                               for fn, is_collection in
+                                   self.app.metaindexconf.find_all_sidecar_files(old_path)
+                               if not is_collection]
+
+            # see if there is any other sidecar file in place already
+            sidecar, is_collection, store = \
+                self.app.metaindexconf.resolve_sidecar_for(new_path)
+
+            logger.debug("trying to put %s metadata tags into %s",
+                         len(writeable_metadata), sidecar)
+
+            if len(direct_sidecars) > 0:
+                for sidecar in direct_sidecars:
+                    new_sidecar = new_path.parent / (new_path.stem + sidecar.suffix)
+                    try:
+                        shutil.copy2(sidecar, new_sidecar)
+                    except OSError as exc:
+                        logger.error("Failed to copy %s to %s: %s",
+                                     sidecar, new_sidecar, exc)
+                        self.app.error("Could not copy all sidecar files. See logfile for details")
+                        prevent_deletions = True
+
+            elif sidecar.exists():
+                assert is_collection
+                existing = store.get_for_collection(sidecar, '')
+                existing = utils.collection_meta_as_writable(existing, new_path.parent)
+                existing[str(new_path.name)] = writeable_metadata
+                logger.debug("Writing collection %s into %s",
+                             existing, sidecar)
+                store.store(existing, sidecar)
+
+            elif len(preferred_stores) > 0:
+                store = preferred_stores[0]
+                sidecar = new_path.parent / (new_path.stem + store.SUFFIX)
+                logger.debug("writing into direct sidecar %s: %s", sidecar, writeable_metadata)
+                store.store(writeable_metadata, sidecar)
+
+            else:
+                logger.error("Could not write extra metadata of %s (copy of %s): "
+                             "no available stores",
+                             new_path, old_path)
+                prevent_deletions = True
+
+        if len(inserts) > 0:
+            self.app.cache.bulk_insert(inserts)
+
+        if len(moved_directories) > 0:
+            self.app.cache.bulk_rename([(old, new, True) for old, new in moved_directories])
+
+        deletes = []
+        if not prevent_deletions:
+            for old_path, _ in moved_files:
+                for sidecar, is_collection in self.app.metaindexconf.find_all_sidecar_files(old_path):
+                    if is_collection:
+                        # TODO - remove the file-specific entries from the sidecar file
+                        continue
+                    try:
+                        sidecar.unlink()
+                    except OSError as exc:
+                        self.app.error(f"Failed to delete sidecar file '{sidecar.name}': {exc}")
+                deletes.append(old_path)
+
+        if not prevent_deletions and len(deletes) > 0:
+            self.app.cache.forget(deletes)
+
+        if self.observer is not None:
+            self.observer.wake_up()
+
     def mkdir(self, name):
         if len(name) == '':
             self.app.error("No folder name given")
@@ -208,13 +398,14 @@ class FilePanel(ListPanel):
         self.reload()
 
     def delete(self, items):
-        self.app.confirm(f"Are you sure you want to delete '{', '.join([item.name for item in items])}'?",
+        itemnames = ', '.join([item.name for item in items])
+        self.app.confirm(f"Are you sure you want to delete '{itemnames}'?",
                          'n',
                          lambda: self.run_blocking(self.do_delete, items))
 
     def do_delete(self, context, items):
         for counter, item in enumerate(items):
-            if len(items) > 1:
+            if len(items) > 1 and context is not None:
                 context.progress((counter, len(items)))
             if item.is_dir():
                 try:
@@ -230,7 +421,7 @@ class FilePanel(ListPanel):
                     self.app.error(f"Failed to delete '{item.name}': {exc}")
                     logger.error("Failed to delete %s: %s", item, exc)
 
-                if delete_sidecars:
+                if delete_sidecars and item.stem not in self.metaindexfilenames:
                     for sidecar, is_collection in self.app.metaindexconf.find_all_sidecar_files(item):
                         if is_collection:
                             # TODO: drop *specific* entries from this file
@@ -279,7 +470,7 @@ class FilePanel(ListPanel):
                 logger.error(f"Failed to rename {sidecar} to {new_sidecar}: {exc}")
 
         for sidecar in rename_inside_sidecars:
-            metadata = stores.get_for_collection(sidecar)
+            metadata = stores.get_for_collection(sidecar, '')
             if path not in metadata:
                 continue
             entry = metadata.pop(path)
@@ -310,7 +501,7 @@ class FilePanel(ListPanel):
     def do_paint_item(self, y, x, maxwidth, is_selected, item):
         attr = item.attr
 
-        if is_selected:
+        if is_selected and self.app.current_panel is self:
             attr |= curses.A_STANDOUT
 
         attr |= colors.attr((item.fg, item.bg))
@@ -321,12 +512,20 @@ class FilePanel(ListPanel):
         if item in self.multi_selection:
             text = "✔ " + text
             attr |= curses.A_BOLD
-        textlen = len(text)
         try:
             self.win.addstr(y, x, " "*maxwidth, attr)
-        except:
+        except curses.error:
             pass
-        self.win.addstr(y, x, text[:maxwidth], attr)
+        try:
+            self.win.addstr(y, x, text[:maxwidth], attr)
+        except curses.error:
+            pass
+
+    def handle_scrolling_key(self, key):
+        handled, must_repaint, must_clear = super().handle_scrolling_key(key)
+        if handled:
+            self.cursor_cache[self.path] = self.cursor
+        return handled, must_repaint, must_clear
 
     def jump_to(self, item, path=None):
         logger.debug(f"Jump to {path}/{item}")
@@ -349,9 +548,21 @@ class FilePanel(ListPanel):
             if not self.is_busy:
                 self.paint()
 
+    def on_focus_lost(self):
+        if self.selected_item is not None:
+            logger.debug("lost focus, repaint line %s", self.selected_item)
+            self.paint_item(self.selected_item)
+            self.win.noutrefresh()
+
     def on_focus(self):
-        if pathlib.Path.cwd() != self.path:
+        try:
+            here = pathlib.Path.cwd()
+        except OSError:
+            here = self.path
+        if here != self.path:
             os.chdir(self.path)
+        if self.items is not None and self.selected_item is not None:
+            self.paint_item(self.selected_item)
 
     def on_close(self):
         logger.debug(f"Closing {self}")
@@ -399,25 +610,54 @@ class MakeFolder(command.Command):
         context.panel.mkdir(" ".join(args))
 
 
-@command.registered_command
-class PasteItem(command.Command):
-    """Paste content of clipboard here"""
-    NAME = 'paste'
-    ACCEPT_IN = (FilePanel,)
-
-    def execute(self, context, clipboard=None):
-        if clipboard is None:
-            clipboard = context.application.DEFAULT_CLIPBOARD
-        # TODO: implement this
-        context.application.error("Not implemented yet")
-
-
 @command.registered_command
 class ChangeDirectory(command.Command):
     """Change directory"""
     NAME = 'cd'
     ACCEPT_IN = (FilePanel,)
 
+    def completion_options(self, context, *args):
+        filepanel = context.application.previous_focus
+        if not isinstance(filepanel, FilePanel):
+            return []
+        basedir = filepanel.path
+
+        if basedir is None:
+            return []
+
+        userinput = None
+        selected_dir = basedir
+        if len(args) > 0 and len(args[0]) > 0:
+            userinput = pathlib.Path(args[0])
+            if userinput.is_absolute():
+                basedir = userinput.parent
+            if (basedir / userinput).is_dir():
+                selected_dir = basedir / userinput
+            else:
+                selected_dir = (basedir / userinput).parent
+
+        if selected_dir.is_dir():
+            options = []
+            for path in sorted(selected_dir.iterdir()):
+                if not path.is_dir():
+                    continue
+                useful = userinput is None or \
+                         (basedir / userinput).is_dir() or \
+                         userinput.parts[-1] in path.name or \
+                         str(userinput) in path.name
+
+                if not useful:
+                    continue
+                # (userinput is None or userinput.is_dir() or userinput.name in path.name) and
+                new_path = path
+                if userinput is None or not userinput.is_absolute():
+                    new_path = path.relative_to(basedir)
+                new_path = str(new_path) + '/'
+                options.append(Completion.Suggestion(new_path, path.name))
+            return options
+
+        return []
+
     def execute(self, context, target=None):
         if target is None:
             context.application.error("No path given")
