@@ -1,8 +1,10 @@
 import threading
 import curses
+import traceback
 
-from cursedspace import ScrollPanel, Key
+from cursedspace import ScrollPanel
 
+from metaindexmanager import shared
 from metaindexmanager.utils import logger
 
 
@@ -45,7 +47,7 @@ class BlockingTask(ScrollPanel):
         self.indicator_idx = 0
         self.autoresize()
 
-    def handle_key(self, key):
+    def handle_key(self, _):
         return False
 
     def check(self):
@@ -65,7 +67,7 @@ class BlockingTask(ScrollPanel):
     def paint(self, clear=False):
         super().paint(clear)
 
-        height, width = self.dim
+        _, width = self.dim
 
         text = self._title[:width-3]
         self.win.addstr(1, 1, text, curses.A_BOLD)
@@ -145,10 +147,12 @@ class BlockingTask(ScrollPanel):
 
     def do_run(self):
         self.app.callbacks.put((self, self.check))
-        try:
-            self.target()
-        except Exception as exc:
-            logger.error(f"Blocking task failed: {exc}")
+        if self.target is not None:
+            try:
+                self.target()
+            except Exception as exc:
+                logger.error(f"Blocking task failed: {exc}")
+                logger.debug(''.join(traceback.format_tb(exc.__traceback__)))
         self.app.callbacks.put((self, self.destroy))
 
 
@@ -216,12 +220,16 @@ class ListPanel(ScrollPanel):
         changed = False
 
         if name == OPTION_PAGING or name is None:
-            new_value = self.app.configuration.bool('all', OPTION_PAGING, str(self.SCROLL_PAGING))
+            new_value = self.app.configuration.bool(shared.ALL_SCOPE,
+                                                    OPTION_PAGING,
+                                                    str(self.SCROLL_PAGING))
             changed = new_value != self.SCROLL_PAGING
             self.SCROLL_PAGING = new_value
 
         if name == OPTION_SCROLL_MARGIN or name is None:
-            new_value = self.app.configuration.number('all', OPTION_SCROLL_MARGIN, str(self.SCROLL_MARGIN))
+            new_value = self.app.configuration.number(shared.ALL_SCOPE,
+                                                      OPTION_SCROLL_MARGIN,
+                                                      str(self.SCROLL_MARGIN))
             if new_value is not None and new_value >= 0:
                 changed = changed or new_value != self.SCROLL_MARGIN
                 self.SCROLL_MARGIN = new_value
@@ -230,17 +238,15 @@ class ListPanel(ScrollPanel):
             self.scroll()
             self.paint(True)
 
-    def focus(self):
-        retval = super().focus()
-        self.on_focus()
-        return retval
-
     def on_close(self):
         pass
 
     def on_focus(self):
         pass
 
+    def on_focus_lost(self):
+        pass
+
 
 def extract_key_sequence(application, commandname):
     return [keys
